% bg.m: Maximizes (minimizes negative) log likelihood function for
%         the Barro-Gordon model when the natural rate of unemployment
%         follows an AR(1) in first differences.
%
%       When maximizing the log likelihood function, parameterizes the
%         cholesky factorization of the covariance matrix, instead of
%         the covariance matrix itself, in order to insure positive
%         definiteness. Parameterizes the autoregressive parameter
%         as lam=lamtr/(1+abs(lamtr)) to insure that it is less than
%         one in absolute value. The log likelihood function with
%         transformed parameters is contained in bgllfn.m.
%
%       When calculating standard errors, uses the original
%         parameterization. The log likelihood function without
%         transformed parameters is contained in bgllfnse.m.
%
%       The untransformed parameter estimates are reported in
%         tstar = [ alpha biga lam sige sign sigen ]'. Their
%         standard errors are reported in sevec.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data

  global ut pit

  load u.dat;
  load def.dat;

  ut = u;
  pit = def;

% adjust sample size if necessary

%  T = length(ut);

%  ut = ut(41:T);

%  pit = pit(41:T);

% set starting values

  alpha = 0.5;
  biga = 0.5;
  lamtr = 1;
  chole = 1;
  choln = 1;
  cholen = 0;

  bigtheto = [ alpha biga lamtr chole choln cholen ]';

% maximize likelihood

  options(1) = 1;

  thetstar = fminu('bgllfn',bigtheto,options);

% find standard errors

  alpha = thetstar(1);
  biga = thetstar(2);
  lam = thetstar(3)/(1+thetstar(3));

  cholsig = [ thetstar(4) 0 ; thetstar(6) thetstar(5) ];
  sig = cholsig*cholsig';

  sige = sqrt(sig(1,1));
  sign = sqrt(sig(2,2));
  sigen = sig(1,2);

  tstar = [ alpha biga lam sige sign sigen ]';

  fstar = bgllfnse(tstar);

  epsmat = 1e-6*eye(6);

  hessvec = zeros(6,1);

  for i = 1:6

    hessvec(i) = bgllfnse(tstar+epsmat(:,i));

  end

  hessmat = zeros(6,6);

  for i = 1:6

    for j = 1:6

      hessmat(i,j) = (bgllfnse(tstar+epsmat(:,i)+epsmat(:,j)) ...
                        -hessvec(i)-hessvec(j)+fstar)/1e-12;

    end

  end

  sevec = sqrt(diag(inv(hessmat)));